require 'puppet/resource_api'

# rubocop:disable Layout/EmptyLinesAroundClassBody

# class Puppet::Provider::PowerstoreImportSession::PowerstoreImportSession
class Puppet::Provider::PowerstoreImportSession::PowerstoreImportSession
  def canonicalize(_context, resources)
    # nout to do here but seems we need to implement it
    resources
  end

  def get(context)
    context.debug('Entered get')
    hash = self.class.fetch_all_as_hash(context)
    context.debug("Completed get, returning hash #{hash}")
    hash
  end

  def set(context, changes, noop: false)
    context.debug('Entered set')

    changes.each do |name, change|
      context.debug("set change with #{name} and #{change}")
      # FIXME: key[:name] below hardwires the unique key of the resource to be :name
      is = change.key?(:is) ? change[:is] : get(context).find { |key| key[:name] == name }
      should = change[:should]

      is = { name: name, ensure: 'absent' } if is.nil?
      should = { name: name, ensure: 'absent' } if should.nil?

      if is[:ensure].to_s == 'absent' && should[:ensure].to_s == 'present'
        create(context, name, should) unless noop
      elsif is[:ensure].to_s == 'present' && should[:ensure].to_s == 'absent'
        context.deleting(name) do
          # FIXME: hardwired
          should[:id] = is[:id]
          delete(context, should) unless noop
        end
      elsif is[:ensure].to_s == 'absent' && should[:ensure].to_s == 'absent'
        context.failed(name, message: 'Unexpected absent to absent change')
      elsif is[:ensure].to_s == 'present' && should[:ensure].to_s == 'present'
        # FIXME: hardwired
        should[:id] = is[:id]
        update(context, name, should)
      end
    end
  end

  def create(context, name, should)
    context.creating(name) do
      new_hash = build_create_hash(should)
      new_hash.delete('id')
      response = self.class.invoke_create(context, should, new_hash)

      raise("Create failed.  Response is #{response} and body is #{response.body}") unless response.is_a? Net::HTTPSuccess
      should[:ensure] = 'present'
      Puppet.info('Added :ensure to property hash')
    end
  rescue StandardError => ex
    Puppet.alert("Exception during create. The state of the resource is unknown.  ex is #{ex} and backtrace is #{ex.backtrace}")
    raise
  end

  def update(context, name, should)
    context.updating(name) do
      new_hash = build_update_hash(should)
      new_hash.delete('id')
      response = self.class.invoke_update(context, should, new_hash)

      raise("Update failed. The state of the resource is unknown.  Response is #{response} and body is #{response.body}") unless response.is_a? Net::HTTPSuccess
      should[:ensure] = 'present'
      Puppet.info('Added :ensure to property hash')
    end
  rescue StandardError => ex
    Puppet.alert("Exception during update. ex is #{ex} and backtrace is #{ex.backtrace}")
    raise
  end

  def build_create_hash(resource)
    import_session = {}
    import_session['automatic_cutover'] = resource[:automatic_cutover] unless resource[:automatic_cutover].nil?
    import_session['description'] = resource[:description] unless resource[:description].nil?
    import_session['name'] = resource[:name] unless resource[:name].nil?
    import_session['protection_policy_id'] = resource[:protection_policy_id] unless resource[:protection_policy_id].nil?
    import_session['remote_system_id'] = resource[:remote_system_id] unless resource[:remote_system_id].nil?
    import_session['scheduled_timestamp'] = resource[:scheduled_timestamp] unless resource[:scheduled_timestamp].nil?
    import_session['source_resource_id'] = resource[:source_resource_id] unless resource[:source_resource_id].nil?
    import_session['volume_group_id'] = resource[:volume_group_id] unless resource[:volume_group_id].nil?
    import_session
  end
  def build_update_hash(resource)
    import_session = {}
    import_session['scheduled_timestamp'] = resource[:scheduled_timestamp] unless resource[:scheduled_timestamp].nil?
    import_session
  end
  # rubocop:disable Lint/UnusedMethodArgument
  def build_delete_hash(resource)
    import_session = {}
    import_session
  end
  # rubocop:enable Lint/UnusedMethodArgument

  def build_hash(resource)
    import_session = {}
    import_session['automatic_cutover'] = resource[:automatic_cutover] unless resource[:automatic_cutover].nil?
    import_session['average_transfer_rate'] = resource[:average_transfer_rate] unless resource[:average_transfer_rate].nil?
    import_session['current_transfer_rate'] = resource[:current_transfer_rate] unless resource[:current_transfer_rate].nil?
    import_session['description'] = resource[:description] unless resource[:description].nil?
    import_session['destination_resource_id'] = resource[:destination_resource_id] unless resource[:destination_resource_id].nil?
    import_session['destination_resource_type'] = resource[:destination_resource_type] unless resource[:destination_resource_type].nil?
    import_session['destination_resource_type_l10n'] = resource[:destination_resource_type_l10n] unless resource[:destination_resource_type_l10n].nil?
    import_session['error'] = resource[:error] unless resource[:error].nil?
    import_session['estimated_completion_timestamp'] = resource[:estimated_completion_timestamp] unless resource[:estimated_completion_timestamp].nil?
    import_session['id'] = resource[:id] unless resource[:id].nil?
    import_session['last_update_timestamp'] = resource[:last_update_timestamp] unless resource[:last_update_timestamp].nil?
    import_session['name'] = resource[:name] unless resource[:name].nil?
    import_session['parent_session_id'] = resource[:parent_session_id] unless resource[:parent_session_id].nil?
    import_session['progress_percentage'] = resource[:progress_percentage] unless resource[:progress_percentage].nil?
    import_session['protection_policy_id'] = resource[:protection_policy_id] unless resource[:protection_policy_id].nil?
    import_session['remote_system_id'] = resource[:remote_system_id] unless resource[:remote_system_id].nil?
    import_session['scheduled_timestamp'] = resource[:scheduled_timestamp] unless resource[:scheduled_timestamp].nil?
    import_session['source_resource_id'] = resource[:source_resource_id] unless resource[:source_resource_id].nil?
    import_session['state'] = resource[:state] unless resource[:state].nil?
    import_session['state_l10n'] = resource[:state_l10n] unless resource[:state_l10n].nil?
    import_session['volume_group_id'] = resource[:volume_group_id] unless resource[:volume_group_id].nil?
    import_session
  end

  def self.build_key_values
    key_values = {}
    key_values['api-version'] = 'assets'
    key_values
  end

  def delete(context, should)
    new_hash = build_delete_hash(should)
    response = self.class.invoke_delete(context, should, new_hash)
    raise("Delete failed.  The state of the resource is unknown.  Response is #{response} and body is #{response.body}") unless response.is_a? Net::HTTPSuccess
    should[:ensure] = 'absent'
    Puppet.info "Added 'absent' to property_hash"
  rescue StandardError => ex
    Puppet.alert("Exception during destroy. ex is #{ex} and backtrace is #{ex.backtrace}")
    raise
  end


  def self.invoke_list_all(context, resource = nil, body_params = nil)
    key_values = build_key_values
    Puppet.info('Calling operation import_session_collection_query')
    path_params = {}
    query_params = {}
    header_params = {}
    header_params['User-Agent'] = ''

    op_params = [
    ]
    op_params.each do |i|
      inquery = i[:inquery]
      name    = i[:name]
      paramalias = i[:paramalias]
      name_snake = i[:namesnake]
      if inquery == 'query'
        query_params[name] = key_values[name] unless key_values[name].nil?
        query_params[name] = ENV["powerstore_#{name_snake}"] unless ENV["powerstore_#{name_snake}"].nil?
        query_params[name] = resource[paramalias.to_sym] unless resource.nil? || resource[paramalias.to_sym].nil?
      else
        path_params[name_snake.to_sym] = key_values[name] unless key_values[name].nil?
        path_params[name_snake.to_sym] = ENV["powerstore_#{name_snake}"] unless ENV["powerstore_#{name_snake}"].nil?
        path_params[name_snake.to_sym] = resource[paramalias.to_sym] unless resource.nil? || resource[paramalias.to_sym].nil?
      end
    end
    context.transport.call_op(path_params, query_params, header_params, body_params, '/import_session', 'Get', 'application/json')
  end


  def self.invoke_create(context, resource = nil, body_params = nil)
    key_values = build_key_values
    Puppet.info('Calling operation import_session_create')
    path_params = {}
    query_params = {}
    header_params = {}
    header_params['User-Agent'] = ''

    op_params = [
      op_param('automatic_cutover', 'body', 'automatic_cutover', 'automatic_cutover'),
      op_param('description', 'body', 'description', 'description'),
      op_param('name', 'body', 'name', 'name'),
      op_param('protection_policy_id', 'body', 'protection_policy_id', 'protection_policy_id'),
      op_param('remote_system_id', 'body', 'remote_system_id', 'remote_system_id'),
      op_param('scheduled_timestamp', 'body', 'scheduled_timestamp', 'scheduled_timestamp'),
      op_param('source_resource_id', 'body', 'source_resource_id', 'source_resource_id'),
      op_param('volume_group_id', 'body', 'volume_group_id', 'volume_group_id'),
    ]
    op_params.each do |i|
      inquery = i[:inquery]
      name    = i[:name]
      paramalias = i[:paramalias]
      name_snake = i[:namesnake]
      if inquery == 'query'
        query_params[name] = key_values[name] unless key_values[name].nil?
        query_params[name] = ENV["powerstore_#{name_snake}"] unless ENV["powerstore_#{name_snake}"].nil?
        query_params[name] = resource[paramalias.to_sym] unless resource.nil? || resource[paramalias.to_sym].nil?
      else
        path_params[name_snake.to_sym] = key_values[name] unless key_values[name].nil?
        path_params[name_snake.to_sym] = ENV["powerstore_#{name_snake}"] unless ENV["powerstore_#{name_snake}"].nil?
        path_params[name_snake.to_sym] = resource[paramalias.to_sym] unless resource.nil? || resource[paramalias.to_sym].nil?
      end
    end
    context.transport.call_op(path_params, query_params, header_params, body_params, '/import_session', 'Post', 'application/json')
  end


  def self.invoke_update(context, resource = nil, body_params = nil)
    key_values = build_key_values
    Puppet.info('Calling operation import_session_modify')
    path_params = {}
    query_params = {}
    header_params = {}
    header_params['User-Agent'] = ''

    op_params = [
      op_param('id', 'path', 'id', 'id'),
      op_param('scheduled_timestamp', 'body', 'scheduled_timestamp', 'scheduled_timestamp'),
    ]
    op_params.each do |i|
      inquery = i[:inquery]
      name    = i[:name]
      paramalias = i[:paramalias]
      name_snake = i[:namesnake]
      if inquery == 'query'
        query_params[name] = key_values[name] unless key_values[name].nil?
        query_params[name] = ENV["powerstore_#{name_snake}"] unless ENV["powerstore_#{name_snake}"].nil?
        query_params[name] = resource[paramalias.to_sym] unless resource.nil? || resource[paramalias.to_sym].nil?
      else
        path_params[name_snake.to_sym] = key_values[name] unless key_values[name].nil?
        path_params[name_snake.to_sym] = ENV["powerstore_#{name_snake}"] unless ENV["powerstore_#{name_snake}"].nil?
        path_params[name_snake.to_sym] = resource[paramalias.to_sym] unless resource.nil? || resource[paramalias.to_sym].nil?
      end
    end
    context.transport.call_op(path_params, query_params, header_params, body_params, '/import_session/%{id}', 'Patch', 'application/json')
  end


  def self.invoke_delete(context, resource = nil, body_params = nil)
    key_values = build_key_values
    Puppet.info('Calling operation import_session_delete')
    path_params = {}
    query_params = {}
    header_params = {}
    header_params['User-Agent'] = ''

    op_params = [
      op_param('id', 'path', 'id', 'id'),
    ]
    op_params.each do |i|
      inquery = i[:inquery]
      name    = i[:name]
      paramalias = i[:paramalias]
      name_snake = i[:namesnake]
      if inquery == 'query'
        query_params[name] = key_values[name] unless key_values[name].nil?
        query_params[name] = ENV["powerstore_#{name_snake}"] unless ENV["powerstore_#{name_snake}"].nil?
        query_params[name] = resource[paramalias.to_sym] unless resource.nil? || resource[paramalias.to_sym].nil?
      else
        path_params[name_snake.to_sym] = key_values[name] unless key_values[name].nil?
        path_params[name_snake.to_sym] = ENV["powerstore_#{name_snake}"] unless ENV["powerstore_#{name_snake}"].nil?
        path_params[name_snake.to_sym] = resource[paramalias.to_sym] unless resource.nil? || resource[paramalias.to_sym].nil?
      end
    end
    context.transport.call_op(path_params, query_params, header_params, body_params, '/import_session/%{id}', 'Delete', 'application/json')
  end




  def self.invoke_get_one(context, resource = nil, body_params = nil)
    key_values = build_key_values
    Puppet.info('Calling operation import_session_instance_query')
    path_params = {}
    query_params = {}
    header_params = {}
    header_params['User-Agent'] = ''

    op_params = [
      op_param('id', 'path', 'id', 'id'),
    ]
    op_params.each do |i|
      inquery = i[:inquery]
      name    = i[:name]
      paramalias = i[:paramalias]
      name_snake = i[:namesnake]
      if inquery == 'query'
        query_params[name] = key_values[name] unless key_values[name].nil?
        query_params[name] = ENV["powerstore_#{name_snake}"] unless ENV["powerstore_#{name_snake}"].nil?
        query_params[name] = resource[paramalias.to_sym] unless resource.nil? || resource[paramalias.to_sym].nil?
      else
        path_params[name_snake.to_sym] = key_values[name] unless key_values[name].nil?
        path_params[name_snake.to_sym] = ENV["powerstore_#{name_snake}"] unless ENV["powerstore_#{name_snake}"].nil?
        path_params[name_snake.to_sym] = resource[paramalias.to_sym] unless resource.nil? || resource[paramalias.to_sym].nil?
      end
    end
    context.transport.call_op(path_params, query_params, header_params, body_params, '/import_session/%{id}', 'Get', 'application/json')
  end


  def self.fetch_all_as_hash(context)
    items = fetch_all(context)
    if items
      items.map { |item|
        hash = {
          automatic_cutover: item['automatic_cutover'],
          average_transfer_rate: item['average_transfer_rate'],
          current_transfer_rate: item['current_transfer_rate'],
          description: item['description'],
          destination_resource_id: item['destination_resource_id'],
          destination_resource_type: item['destination_resource_type'],
          destination_resource_type_l10n: item['destination_resource_type_l10n'],
          error: item['error'],
          estimated_completion_timestamp: item['estimated_completion_timestamp'],
          id: item['id'],
          last_update_timestamp: item['last_update_timestamp'],
          name: item['name'],
          parent_session_id: item['parent_session_id'],
          progress_percentage: item['progress_percentage'],
          protection_policy_id: item['protection_policy_id'],
          remote_system_id: item['remote_system_id'],
          scheduled_timestamp: item['scheduled_timestamp'],
          source_resource_id: item['source_resource_id'],
          state: item['state'],
          state_l10n: item['state_l10n'],
          volume_group_id: item['volume_group_id'],
          ensure: 'present',
        }
        Puppet.debug("Adding to collection: #{item}")
        hash
      }.compact
    else
      []
    end
  rescue StandardError => ex
    Puppet.alert("ex is #{ex} and backtrace is #{ex.backtrace}")
    raise
  end

  def self.deep_delete(hash_item, tokens)
    if tokens.size == 1
      if hash_item.is_a?(Array)
        hash_item.map! { |item| deep_delete(item, tokens) }
      else
        hash_item.delete(tokens[0]) unless hash_item.nil? || hash_item[tokens[0]].nil?
      end
    elsif hash_item.is_a?(Array)
      hash_item.map! { |item| deep_delete(item, tokens[1..-1]) }
    else
      hash_item[tokens.first] = deep_delete(hash_item[tokens.first], tokens[1..-1]) unless hash_item.nil? || hash_item[tokens[0]].nil?
    end
    hash_item
  end

  def self.fetch_all(context)
    response = invoke_list_all(context)
    return unless response.is_a? Net::HTTPSuccess
    body = JSON.parse(response.body)
    body # ["value"] if body.is_a? Array # and body.key? "value"
  end

  def self.authenticate(_path_params, _query_params, _header_params, _body_params)
    true
  end


  def exists?
    return_value = @property_hash[:ensure] && @property_hash[:ensure] != 'absent'
    Puppet.info("Checking if resource #{name} of type <no value> exists, returning #{return_value}")
    return_value
  end

  def self.add_keys_to_request(request, hash)
    hash.each { |x, v| request[x] = v } if hash
  end

  def self.to_query(hash)
    if hash
      return_value = hash.map { |x, v| "#{x}=#{v}" }.reduce { |x, v| "#{x}&#{v}" }
      return return_value unless return_value.nil?
    end
    ''
  end

  def self.op_param(name, inquery, paramalias, namesnake)
    { name: name, inquery: inquery, paramalias: paramalias, namesnake: namesnake }
  end


end
